#!/usr/bin/env python

"""PythonicaLexer takes a stream of text, and converts it into a stream
of tokens appropriate for a parser to digest.

Danny Yoo (dyoo@hkn.eecs.berkeley.edu)


We use Greg Ewing's very cool Plex lexical analyzer at:

    http://www.cosc.canterbury.ac.nz/~greg

to do all of the heavy lifting for us.  I have not played with
Mathematica enough to know if this is all correct, so I expect someone
to help me fix the bugs here.


By the way, this is NOT meant to be imported globally: if anything,
the only things that are safe to import are:

    makeScanner
    EOF
    lexicon

Everything else is messy.  *grin*
"""


from Plex import *
import StringIO


def makeLexer(s):
    """Given a string, returns a Lexer that tries to parse s."""
    return PlexLexicalWrapper(Scanner(lexicon, StringIO.StringIO(s), s))


######################################################################
class PlexLexicalWrapper:
    """A small wrapper around a Plex Scanner to let it work more
    nicely with my parser.  It will allow me to do 1-token
    lookahead."""
    def __init__(self, scanner):
        self.scanner = scanner
        self.lookahead = self.scanner.read()

    def hasMoreTokens(self):
        return self.lookahead != (None, '')

    def peek(self):
        return self.lookahead

    def eat(self, token):
        if token != self.lookahead:
            raise LexerError, ("Token " + str(token) + " does not match.")
        self.lookahead = self.scanner.read()
######################################################################



EOF = (None, '')


_letter = Range("AZaz")
_digit = Range("09")
_name = _letter + Rep(_letter | _digit)
_identifier = _name | Str("%")
_integer = Rep1(_digit)
_space = Any(" \t\n")

_implies = Str("->")
_equality = Str("==")
_unassignment = Str("=.")
_assignment = Str("=")
_ruledefine = Str("->")
_ruleapply = Str("/.")

lexicon = Lexicon([
    (_identifier, 'identifier'),
    (_integer, 'number'),
    (Any("+-*/^=()[]"), TEXT),
    (_implies, TEXT),
    (_equality, TEXT),
    (_unassignment, TEXT),
    (_assignment, TEXT),
    (_ruledefine, TEXT),
    (_ruleapply, TEXT),
    (_space, IGNORE)])
