'''PythonicaLexer.py --- A small lexical analyzer for Pythonica.

Danny Yoo (dyoo@hkn.eecs.berkeley.edu)

We use classes from the really nice spark parser to generate this
lexer.

'''

from spark import GenericScanner

class LexerError(Exception): pass

def tokenize(s):
    try:
        scanner = SimpleScanner2()
        return scanner.tokenize(s)
    except Exception, e:
        raise LexerError, e

class SimpleScanner1(GenericScanner):
    'Scanner to take care of simple tokens'
    def __init__(self):
        GenericScanner.__init__(self)

    def tokenize(self, input):
        self.rv = []
        GenericScanner.tokenize(self, input)
        return self.rv

    def t_whitespace(self, s):
        r' \s+ '
        pass # skip whitespace

    def t_singleop(self, s):
        r' \+ | \- | \= | \/ | \* | \^ '
        self.rv.append(Token(type=s))

    def t_identifier(self, s):
        r' \% | [A-Za-z][A-Za-z0-9\_]* '
        self.rv.append(Token(type='identifier', attr=s))

    def t_integer(self, s):
        r' \d+ '
        self.rv.append(Token(type='number', attr=s))

    def t_parens(self, s):
        r' \( | \)'
        if s == '(':
            self.rv.append(Token(type='('))
        else:
            self.rv.append(Token(type=')'))

    def t_braces(self, s):
        r' \[ | \]'
        if s == '[':
            self.rv.append(Token(type='['))
        else:
            self.rv.append(Token(type=']'))
    

class SimpleScanner2(SimpleScanner1):
    '''Scanner to handle certain longer tokens that should
    take precedence over those in SimpleScanner1.'''
    def t_floating(self, s):
        r' \d+ \. \d+ '
        self.rv.append(Token(type='number', attr=s))
        
    def t_equality(self, s):
        r' \=\= '
        self.rv.append(Token(type=s))

    def t_unassignment(self, s):
        r' \=\. '
        self.rv.append(Token(type=s))

    def t_ruledefine(self, s):
        r' \-\>  '
        self.rv.append(Token(type=s))

    def t_ruleapply(self, s):
        r' \/\.  '
        self.rv.append(Token(type=s))


class Token:
    '''Defines a single "token", which has type and attr fields'''
    def __init__(self, type, attr=None):
        self.type = type
        self.attr = attr

    def __cmp__(self, o):
        return cmp(self.type, o)

    def __repr__(self):
        return 'Token(%s, %s)' % (repr(self.type),
                                  repr(self.attr))
